unit MenuLib;

interface

uses Menus;

type
  // If this Returns False, it will abort the Walk
  TProcessMenuItem = function(MenuItem: TMenuItem; Data: Pointer): Boolean of object;

procedure EnablePopup(Sender: TObject; Enable: Boolean);

procedure WalkMenu(Menu: TMenu; ProcessItem: TProcessMenuItem; Data: Pointer);
function WalkMenuItem(MenuItem: TMenuItem; ProcessItem: TProcessMenuItem; Data: Pointer): Boolean;

implementation

procedure EnablePopup(Sender: TObject; Enable: Boolean);
var
  i: Integer;
begin
  with Sender as TPopupMenu do
    for i := 0 to Items.Count-1 do
      Items[i].Enabled := Enable;
end;

// This Procedure Will Walk an Entire Popup Menu and Call ProcessItem for Each Menu Item
// if ProcessItem returns False, it will abort immediately
procedure WalkMenu(Menu: TMenu; ProcessItem: TProcessMenuItem; Data: Pointer);
var
  i: Integer;
begin
  with Menu do
    for i := 0 to Items.Count - 1 do
    begin
     if not (ProcessItem(Items[i], Data) and
             WalkMenuItem(Items[i], ProcessItem, Data)) then
        Break; // Abort if ProcessItem or WalkMenuItem Returns False
    end;
end;

// This function Will Walk all the Children of a MenuItem and Call ProcessItem for each one
// if ProcessItem returns False, it will abort immediately and return false
function WalkMenuItem(MenuItem: TMenuItem; ProcessItem: TProcessMenuItem; Data: Pointer): Boolean;
var
  i: Integer;
begin
  Result := True; // Default to True

  with MenuItem do
    for i := 0 to Count - 1 do // Process All Child Menu Items
    begin
      Result := ProcessItem(Items[i], Data); // Process Item but...
      if not Result then Exit;         // ... if it Returns False, then abort

      // If there are subitems, recurse...
      if Items[i].Count > 0 then
      begin
        Result := WalkMenuItem(Items[i], ProcessItem, Data);
        if not Result then Exit; // if it Returns False, abort
      end;
    end;
end;

end.
